# Introduction to ggplot2 and the mpg dataset
library(ggplot2)

# Look at the data we're going to use
?mpg
head(mpg)
str(mpg)

# Basic scatterplot
qplot(displ, hwy, data = mpg)

# Aesthetics  ----------------------------------------------------------------

qplot(displ, hwy, data = mpg, colour = class)
qplot(displ, hwy, data = mpg, colour = cyl)
qplot(displ, hwy, data = mpg, shape = factor(cyl))
qplot(displ, hwy, data = mpg, shape = factor(cyl), colour = factor(cyl))

# Faceting  ------------------------------------------------------------------
qplot(displ, hwy, data = mpg)
qplot(displ, hwy, data = mpg) + facet_grid(. ~ cyl)
qplot(displ, hwy, data = mpg) + facet_grid(drv ~ .)
qplot(displ, hwy, data = mpg) + facet_grid(drv ~ cyl)
qplot(displ, hwy, data = mpg) + facet_wrap(~ class)

# Overplotting  --------------------------------------------------------------

qplot(cty, hwy, data = mpg)
qplot(cty, hwy, data = mpg, geom = "jitter")
qplot(cty, hwy, data = mpg, geom = "jitter", colour = year)
qplot(cty, hwy, data = mpg, geom = "jitter", colour = class)
qplot(cty, hwy, data = mpg, geom = "jitter", colour = class) +
  geom_smooth(method = "lm")

# Reordering and boxplots ----------------------------------------------------

qplot(class, hwy, data = mpg)
qplot(reorder(class, hwy), hwy, data = mpg)
qplot(reorder(class, hwy), hwy, data = mpg, geom = "jitter")
qplot(reorder(class, hwy), hwy, data = mpg, geom = "boxplot")
qplot(reorder(class, hwy), hwy, data = mpg, geom = c("jitter", "boxplot"))

# Bar charts and histograms --------------------------------------------------

# With only one variable, qplot guesses that
# you want a bar chart or histogram
qplot(cut, data = diamonds)

qplot(carat, data = diamonds)
qplot(carat, data = diamonds, binwidth = 1)
qplot(carat, data = diamonds, binwidth = 0.1)
qplot(carat, data = diamonds, binwidth = 0.01)
resolution(diamonds$carat)

last_plot() + xlim(0, 3)

# If you want to add an additional variable to the plot it's better to
# use faceting, rather than aesthetics.  

qplot(depth, data = diamonds, binwidth = 0.2)
# In this plot we can see general trends, but it's hard to get precise
qplot(depth, data = diamonds, binwidth = 0.2, fill = cut) + xlim(55, 70)

# Here the differences really jump out at you
qplot(depth, data = diamonds, binwidth = 0.2) + xlim(55, 70) + 
  facet_wrap(~ cut) 

qplot(carat, data = diamonds, binwidth = 0.1) + facet_wrap(~ color) 

# Instead of displaying counts on the y-axis, you can display sums of 
# other variables:
qplot(cut, data = diamonds, weight = carat)
qplot(cut, data = diamonds, weight = price)

# This is useful for pretabulated data
cuts <- as.data.frame(table(
  cut = diamonds$cut))
qplot(cut, weight = Freq, data = cuts)

# Some times you need to zoom in
qplot(table, data = diamonds, binwidth = 1)

# To zoom in on a plot region use xlim() and ylim()
qplot(table, data = diamonds, binwidth = 1) + xlim(50, 70)
qplot(table, data = diamonds, binwidth = 0.1) + xlim(50, 70)
qplot(table, data = diamonds, binwidth = 0.1) + xlim(50, 70) + ylim(0, 50)

# Note that this type of zooming discards data outside of the plot regions
# See coord_cartesian() for an alternative

# Weighting ------------------------------------------------------------------

qplot(cut, data = diamonds, weight = carat)
qplot(cut, data = diamonds, weight = price)

# Also useful for pretabulated data
cuts <- as.data.frame(table(
  cut = diamonds$cut))
qplot(cut, weight = Freq, data = cuts)

# Box plots ------------------------------------------------------------------

qplot(color, price, data = diamonds, 
  geom = "boxplot")

# Boxplots also useful for continuous data
# But need to specify how to group data into boxes
qplot(table, price, data = diamonds)
qplot(table, price, data = diamonds, geom = "boxplot")
qplot(table, price, data = diamonds, geom = "boxplot", group = round(table))

# Overplotting ---------------------------------------------------------------

qplot(carat, price, data = diamonds, 
  colour = I(alpha("black", 1/255)))

qplot(carat, price, data = diamonds, geom = "bin2d")
qplot(carat, price, data = diamonds, geom = "bin2d", bins = 100)
qplot(carat, price, data = diamonds, geom = "hex")

qplot(carat, price, data = diamonds) + geom_smooth()

# Very basic cleaning
diamonds$x[diamonds$x == 0] <- NA
diamonds$y[diamonds$y == 0] <- NA
diamonds$y[diamonds$y > 12] <- NA

qplot(x, y, data = diamonds)
qplot(x, y, data = diamonds, geom = "bin2d")
qplot(x, y, data = diamonds, geom = "hex")
qplot(x, y, data = diamonds, geom = "bin2d", bins = 100)
qplot(x, y, data = diamonds, geom = "hex", bins = 100)

# Zoom in
qplot(x, y, data = diamonds, geom = "bin2d", bins = 100) + 
  xlim(4,7) + ylim(4,7)
qplot(x, y, data = diamonds, geom = "bin2d", bins = 100) + 
  xlim(4,5) + ylim(4,5)
  
qplot(x, x / y, data = diamonds, geom = "bin2d")
qplot(x, log(x / y), data = diamonds, geom = "bin2d")

clean <- subset(diamonds, abs(log(x / y)) < 0.1)

qplot(x, log(x / y), data = clean, geom = "bin2d")
qplot(x, log(x / y), data = clean, geom = "bin2d", bins = 80)
