# Load the data --------------------------------------------------------------
options(stringsAsFactors = FALSE)

bnames <- read.csv("baby-names.csv")
# OR
bnames <- read.csv(file.choose())

# Extract subsets of interest ------------------------------------------------

subset(bnames, name == "Hadley")

# Save to a new variable instead of printing
hadley <- subset(bnames, name == "Hadley")

# Multiple criteria: & = and, | = or, ! = not
subset(bnames, name == "Hadley" & sex == "boy")

# Other operators: <, <=, >=, >=, !=, %in%
subset(bnames, name == "Hadley" & year < 1910)


# Look at names starting with Had -------------------------------------------
had <- subset(bnames, substr(name, 1, 3) == "Had")

library(ggplot2)
qplot(year, prop, data = had, geom = "line")
qplot(year, prop, data = had, geom = "line", 
  group = name)
qplot(year, prop, data = had, geom = "line", 
  group = interaction(name, sex))

qplot(year, prop, data = had, geom = "line", 
  colour = sex) + facet_wrap(~ name)
last_plot() + geom_point()

# Transformations ------------------------------------------------------------


letter <- function(x, n = 1) {
  if (n < 0) {
    nc <- nchar(x)
    n <- nc + n + 1
  }
  tolower(substr(x, n, n))
}
vowels <- function(x) {
  nchar(gsub("[^aeiouy]", "", tolower(x)))
}

bnames <- transform(bnames,
  first = letter(name, 1),
  last = letter(name, -1),
  vowels = vowels(name),
  length = nchar(name),
  per10000 = 10000 * prop,
  one_per = 1 / prop
)

summarise(bnames, 
  min_length = min(length),
  max_length = max(length)
) 

subset(bnames, length == 2)
subset(bnames, length == 10)


# Calculating group-wise ranks -----------------------------------------------

# Do it for one piece
one <- subset(bnames, sex == "boy" & year == 2008)
one <- transform(one, 
  rank = rank(-prop, ties.method = "first"))
head(one)

# Do it for many pieces with ddply
library(plyr)
bnames <- ddply(bnames, c("sex", "year"), transform, 
  rank = rank(-prop, ties.method = "first"))

# Explore length of names over time ------------------------------------------

# Explore average length

sy <- ddply(bnames, c("sex", "year"), summarise, 
  avg_length = weighted.mean(length, prop))

qplot(year, avg_length, data = sy, colour = sex, 
  geom = "line")
    
# Explore number of names of each length

syl <- ddply(bnames, c("sex", "length", "year"), 
  summarise, prop = sum(prop))
qplot(year, prop, data = syl, colour = sex, 
  geom = "line") + facet_wrap(~ length)

twoletters <- subset(bnames, length == 2)
unique(twoletters$name)
qplot(year, prop, data = twoletters, colour = sex, 
  geom = "line") + facet_wrap(~ name)
  
# The proportion of names in the top 1000 ------------------------------------

sy <- ddply(bnames, c("year","sex"), summarise, 
  prop = sum(prop), 
  npop = sum(prop > 1/1000))

qplot(year, prop, data = sy, colour = sex, geom = "line")
qplot(year, npop, data = sy, colour = sex, geom = "line")

# Distribution of last letters -----------------------------------------------

syl <- ddply(bnames, c("sex", "last", "year"), summarise, 
  prop = sum(prop))
qplot(year, prop, data = syl, colour = sex, geom = "line") + 
  facet_wrap(~ last)
