library(plyr)
library(ggplot2)
library(stringr)

# Load the data --------------------------------------------------------------
options(stringsAsFactors = FALSE)

# Big data tip: read compressed files directly
bnames <- read.csv("baby-names2.csv.bz2")

births <- read.csv("births.csv")
bnames <- join(bnames, births, by = c("year", "sex"))
bnames <- mutate(bnames, n = round(prop * births))

# Compute some new variables

vowels <- function(x) {
  str_length(str_replace_all(tolower(x),
    "[^aeiouy]", ""))
}

bnames <- transform(bnames,
  first = tolower(str_sub(name, 1, 1)),
  last = tolower(str_sub(name, -1, -1)),
  vowels = vowels(name),
  length = nchar(name),
  per10000 = 10000 * prop,
  one_per = 1 / prop
)

# Extract subsets of interest ------------------------------------------------

hadley <- subset(bnames, name == "Hadley")

qplot(year, prop, data = hadley, colour = sex, geom ="line")

# Names that sound like Gabriel
gabi <- subset(bnames, soundex == "G164")
qplot(year, prop, data = gabi)

# Be careful with grouping!
qplot(year, prop, data = gabi, geom = "line")

qplot(year, prop, data = gabi, geom = "line", colour = sex) + 
  facet_wrap(~ name)

qplot(year, prop, data = gabi, geom = "line", colour = sex, 
  group = interaction(sex, name))

# Other data manipulation functions ------------------------------------------

summarise(hadley, 
  total = sum(n), 
  avg = mean(n))

arrange(hadley, desc(prop))

mutate(hadley, rank = rank(desc(prop)))

# Group-wise transformations -------------------------------------------------

# Number of people with a name.  Easy for one name:
hadley <- subset(bnames, name == "Hadley")
sum(hadley$n)

# Or
summarise(hadley, n = sum(n))

# For all names: split, apply and combine.
pieces <- split(bnames, list(bnames$name))

results <- vector("list", length(pieces))
for(i in seq_along(pieces)) {
  piece <- pieces[[i]]
  results[[i]] <- summarise(piece, 
    name = name[1], n = sum(n))
}

result <- do.call("rbind", results)

# Or use ddply
counts <- ddply(bnames, "name", summarise,  
  n = sum(n))

# Even faster is the special purpose count function:
counts <- count(bnames, "name", "n")

# Calculating group-wise ranks

# Do it for one piece
one <- subset(bnames, sex == "boy" & year == 2008)
one <- transform(one, rank = rank(desc(prop), ties.method = "min"))
head(one)

# Do it for many pieces with ddply
library(plyr)
bnames <- ddply(bnames, c("sex", "year"), mutate, 
  rank = rank(desc(prop), ties.method = "min"))
  
# More summaries  ------------------------------------------------------------

# Explore average length

sy <- ddply(bnames, c("sex", "year"), summarise, 
  avg_length = weighted.mean(length, prop))

qplot(year, avg_length, data = sy, colour = sex, geom = "line")
    
# Explore number of names of each length

syl <- ddply(bnames, c("sex", "length", "year"), 
  summarise, prop = sum(prop))
qplot(year, prop, data = syl, colour = sex, geom = "line") + 
  facet_wrap(~ length)

twoletters <- subset(bnames, length == 2)
unique(twoletters$name)
qplot(year, prop, data = twoletters, colour = sex, geom = "line") +
  facet_wrap(~ name)
  
# Number of vowels in a name
vys <- ddply(bnames, c("vowels", "year", "sex"),
  summarise, prop = sum(prop))
qplot(year, prop, data = vys, colour = sex, geom = "line") + 
  facet_wrap(~ vowels)

# The distribution of last letters.
syl <- ddply(bnames, c("sex", "last", "year"), 
  summarise, prop = sum(prop))
qplot(year, prop, data = syl, colour = sex, geom = "line") + 
  facet_wrap(~ last)

# Total proportion of babies with names in the top 1000.
sy <- ddply(bnames, c("year","sex"), summarise, 
  prop = sum(prop))
qplot(year, prop, data = sy, colour = sex, geom = "line")

