library(ggplot2)

# Scale naming convention ----------------------------------------------------

# Default scales
scale_x_continuous()
scale_y_discrete()
scale_colour_discrete()

# Custom scales
scale_colour_hue() 
scale_x_log10()
scale_fill_brewer()

# Scales with parameters
scale_x_continuous("X Label", limits = c(1, 10))
scale_colour_gradient(low = "blue", high = "red")


# Changing ticks and labels --------------------------------------------------

p <- qplot(cyl, displ, data = mpg)

# First argument (name) controls axis label
p + scale_y_continuous("Displacement (l)")
p + scale_x_continuous("Cylinders")

# Breaks and labels control tick marks
p + scale_x_continuous(breaks = c(4, 6, 8))
p + scale_x_continuous(breaks = c(4, 6, 8),
  labels = c("small", "medium", "big"))

# Limits control range of data
p + scale_y_continuous(limits = c(1, 8))
# same as:
p + ylim(1, 8)

### Real example with continuous data 
p <- qplot(carat, price, data = diamonds, geom = "hex")

# First argument (name) controls legend title
p + scale_fill_continuous("Count")

# Breaks and labels control legend keys
p + scale_fill_continuous(breaks = c(1000, 3500, 7000))
p + scale_fill_continuous(breaks = c(0, 4000, 8000))

# Why don't 0 and 8000 have colours?
p + scale_fill_continuous(breaks = c(0, 4000, 8000), 
  limits = c(0, 8000))

# Can use labels to make more human readable
breaks <- c(0, 2000, 4000, 6000, 8000)
labels <- format(breaks, big.mark = ",") 
p + scale_fill_continuous(breaks = breaks, labels = labels,
   limits = c(0, 8000))
   
 
### Basically the same for discrete variables
p <- qplot(color, carat, data = diamonds)
p + scale_x_discrete("Color")

# Except limits is now a character vector
p + scale_x_discrete(limits = c("D", "E", "F"))

# Should work for boxplots too
qplot(color, carat, data = diamonds, 
 geom = "boxplot") + 
 scale_x_discrete(limits = c("D", "E", "F"))

  
# Themes ---------------------------------------------------------------------

# Two built in themes.  The default:
qplot(carat, price, data = diamonds)

# And a theme with a white background:
qplot(carat, price, data = diamonds) + theme_bw()

# Use theme_set if you want it to apply to every
# future plot.
theme_set(theme_bw())

# This is the best way of seeing all the default
# options
theme_bw()
theme_grey()

p <- qplot(displ, hwy, data = mpg) + 
  opts(title = "Bigger engines are less efficient")

# To modify a plot
p 
p + opts(plot.title = 
  theme_text(size = 12, face = "bold"))
p + opts(plot.title = theme_text(colour = "red"))
p + opts(plot.title = theme_text(angle = 45))
p + opts(plot.title = theme_text(hjust = 1))


# Saving your work -----------------------------------------------------------

qplot(carat, price, data = diamonds)
ggsave("diamonds.png")

# Selects graphics device based on extension
ggsave("diamonds.pdf")

# Uses on-screen device size, or override with 
# width & height (to be reproducible)
ggsave("diamonds.png", width = 6, height = 6)

# Outputs last plot by default, override
# with plot:
dplot <- qplot(carat, price, data = diamonds)
ggsave("diamonds.png", plot = dplot)

# Defaults to 300 dpi for png
ggsave("diamonds.png", dpi = 72)

# Clean up -------------------------------------------------------------------
unlink("diamonds.png")
unlink("diamonds.pdf")