library("ggplot2")
library("mgcv")

source("glyphs.r")
source("06-nasa.r")

# Aim: basic modelling methods to remove strong patterns
#      residuals + re-iteration
#
# Tools:
# * lm and gam
# * formula interface
# * predict, resid, summary, anova
# * dlply & ldply
# * pulling infoo out of models
#
# Start with the nasa data then migrate to large sweden or norway data

# How can we tell how temperature is changing?
nasa <- glyphs(nasa, "long", "day", "lat", "surftemp") 
qplot(gx, gy, data = nasa, geom = "path", group = id) + map 

# Introduction to models -----------------------------------------------------

# response ~ predictor
lm(surftemp ~ year, data = nasa)

# use factor to force a categorical predictor
# matrix of dummy variables is made automatically
lm(surftemp ~ factor(month), data = nasa)

mod <- lm(surftemp ~ factor(month), data = nasa)
summary(mod)

# + adds more predictors
lm(surftemp ~ factor(month) + year, data = nasa)

# : makes an interaction
lm(surftemp ~ factor(month):year, data = nasa)

# * adds main effects and interactions
lm(surftemp ~ factor(month) * year, data = nasa)

# A model for each location --------------------------------------------------

# One way to go about it is to fit a model to each location
# dlply is similar to ddply, but instead of creating a data frame,
# it creates a list.
temp_models <- dlply(nasa, c("lat", "long"), function(df) {
  # lm(ozone ~ factor(month) - 1, data = df)
  lm(surftemp ~ factor(month), data = df)
})

temp_models[1:10]
length(temp_models)
temp_models[[1]]
summary(temp_models[[1]])
anova(temp_models[[1]])
coef(temp_models[[1]])
resid(temp_models[[1]])

# Coefficients ---------------------------------------------------------------

# Extract seasonal trends for each location
# ldply is the inverse of dlply - it takes a list and combines the output
# into a data frame

coefs <- ldply(temp_models, coef)

# coef doesn't provide the data in quite the right format so we need to
# create out our own function to do this.
one <- temp_models[[1]]
coef(one)
as.data.frame(coef(one))
cf <- coef(one)
data.frame(coef = names(cf), value = cf)

# Or ...
coefs <- ldply(temp_models, function(x) {
  data.frame(month = 2:12, coef = coef(x)[-1])
})

# Show seasonal patterns
coefs <- glyphs(coefs, "long", "month", "lat", "coef") 
qplot(gx, gy, data = coefs, geom = "line", group = gid) + map 
qplot(gx, gy, data = coefs, geom = "line", group = gid) + 
  geom_path(aes(y = lat), colour = "grey50", alpha = 0.75)

coefs <- glyphs(coefs, "long", "month", "lat", "coef", polar = T) 
qplot(gx, gy, data = coefs, geom = "path", group = gid) + map 
ggplot(coefs, aes(gx, gy, group = gid)) +
  map + 
  geom_point(aes(long, lat), colour = "red", shape = ".") + 
  geom_path()

# But generally it's easier to work with model predictions
# Gives you more flexibility to focus on the components of the model that 
# you're interested in, and typically makes for more interpretable plots

# Predictions ----------------------------------------------------------------

# Let's move to a more sophisticated model where we combine a seasonal pattern
# with a (very) smooth yearly trend.

temp_models <- dlply(nasa, c("lat", "long"), function(df) {
  lm(surftemp ~ year + factor(month), data = df)
})

# Want to explore both long-term and short-term trends
# Key idea is to generate a grid where we'll make predictions

# To explore long-term trend, we'll hold month constant:
year_grid <- expand.grid(year = unique(nasa$year), month = 1)
# To explore seasonal trend, we'll hold year constant:
month_grid <- expand.grid(year = 2000, month = 1:12)

# Then we make for each model at that location

month_preds <- ldply(temp_models, function(mod) {
  month_grid$pred <- predict(mod, newdata = month_grid)
  month_grid
})
year_preds <- ldply(temp_models, function(mod) {
  year_grid$pred <- predict(mod, newdata = year_grid)
  year_grid
})

head(month_preds)
head(year_preds)

month_preds <- glyphs(month_preds, "long", "month", "lat", "pred") 
year_preds <- glyphs(year_preds, "long", "year", "lat", "pred") 

qplot(gx, gy, data = month_preds, geom = "path", group = gid) + map 
qplot(gx, gy, data = year_preds, geom = "path", group = gid) + map 

# Easily adaptable to more sophisticated models, like generalised additive
# models which incorporate a smooth term

# (takes longer because the model is much more complex)
temp_smooth <- dlply(nasa, c("lat", "long"), function(df) {
  gam(surftemp ~ s(day) + factor(month), data = df)
})
day_grid <- expand.grid(day = seq(0, 2161, length = 50), month = 1)
day_preds <- ldply(temp_smooth, function(mod) {
  day_grid$pred <- predict(mod, newdata = day_grid)
  day_grid
})
day_preds <- glyphs(day_preds, "long", "day", "lat", "pred") 
qplot(gx, gy, data = day_preds, geom = "path", group = gid) + map 

# Linear trend is not a good fit!

# Residuals ------------------------------------------------------------------

# Instead of fitting a more complicated model, we could have detected a
# problem by looking at the residuals for each model in temp_smooth. This is
# more complicated to extract because you need pair up the original data with
# the corresponding model

# First we make a list of data frames that matches the models
locs <- dlply(nasa, c("lat", "long"))

# mdply allows us to supply multiple arguments to our summary function - 
# this will pair data with model
resids <- mdply(cbind(d = locs, m = temp_models), function(d, m) {
  d$temp_resid <- d$temp - predict(m, newdata = d)
  d
})

resids <- glyphs(resids, "long", "day", "lat", "temp_resid") 
qplot(gx, gy, data = resids, geom = "line", group = gid) + map

# # WHY ARE THESE NOT THE SAME?!
# one <- subset(nasa, x == 1 & y == 24)
# mod <- lm(temperature ~ factor(month) + year, data = one) 
# one$temp_resid <- resid(mod)
# 
# rone <- subset(resids, x == 1 & y == 24)
# 
# qplot(day, temp_resid, data = one, geom = "line") + 
#   geom_line(data = rone, colour = "red")


# Summary statistics ---------------------------------------------------------

# Allow useful to extract summary statistics from each model
# Challenge: can you create a function that extracts the r-squared from 
#  a given model?  (Hint: look at str(summary(mod)))

rsq <- function(mod) summary(mod)$r.squared
rsqs <- ldply(temp_models, rsq)

qplot(long, lat, data = rsqs, fill = V1, geom = "tile") + map

last_plot() + geom_line(aes(gx, gy, group = gid, fill = NULL), data = resids)

# Models are not fitting well in the equatorial regions.  Why?
