# Anonymous functions --------------------------------------------------------

# Creating an anonymous function
function(x) 3

# Calling an anonymous function
(function(x) 3)()
# Not:
function(x) 3()

# Anonymous functions work just like ordinary 
# functions
formals(function(x = 4) g(x) + h(x))
body(function(x = 4) g(x) + h(x))
environment(function(x = 4) g(x) + h(x))

# Useful for small, one-off tasks that don't
# merit creating a named function

lapply(mtcars, function(x) length(unique(x)))

integrate(function(x) sin(x)^2, 0, pi)

# Closures -------------------------------------------------------------------

x <- 5
f <- function() { 
  y <- 10
  c(x = x, y = y)
}
f()

g <- function() { 
  x <- 20
  y <- 10
  c(x = x, y = y)
}
g()

# What do these functions return?
# How does variable lookup in R work?

x <- 0
y <- 10
f <- function() {
  x <- 1
  function() {  
    y <- 2
    x + y
  }
}

# What does f() return?
# What does f()() mean? What does it do?
# How does it work?


# Closures are useful when you want a function
# that can create a whole class of functions:

power <- function(exponent) {
  function(x) x ^ exponent
}

square <- power(2)
square(2)
square(4)

cube <- power(3)
cube(2)
cube(4)

square

# We can find the environment and its parent
environment(square)
parent.env(environment(square))

# Or inspect objects defined in that environment
ls(environment(square))
get("exponent", environment(square))
environment(square)$exponent
as.list(environment(square))

# Lists of functions ---------------------------------------------------------

compute_mean <- list(
  base = function(x) mean(x),
  sum = function(x) sum(x) / length(x),
  manual = function(x) {
    total <- 0
    n <- length(x)
    for (i in seq_along(x)) {
      total <- total + x[i] / n
    }
    total
  }
)

call_fun <- function(f, ...) f(...)
x <- runif(1e6)
lapply(compute_mean, call_fun, x)
lapply(compute_mean, function(f) system.time(f(x)))


mysummary <- function(x, fs = c(mean, sd), ...) {
  lapply(fs, call_fun, x, ...)
}

make_summary <- function(fs = c(mean, sd)) {
  function(x, ...) {
    lapply(fs, call_fun, x, ...)   
  }
}
mysum <- make_summary(c(sd, IQR, var))