options(digits = 2)

# If you know how long your result will be,
# preallocate the storage
grow <- function() {
  output <- numeric()
  for(i in 1:100) {
    output <- c(output, i ^ 2)
  }
  output  
}

preallocate <- function() {
  
  output <- rep(NA, 10)
  for(i in 1:100) {
    output[i] <- i ^ 2
  }
  output
}

library(microbenchmark)
b <- microbenchmark(grow(), preallocate())
print(b, unit = "eps")

# But you should always vectorise (i.e.
# push loops into pre-written C) where possible
vectorise <- function() (1:100) ^ 2
b <- microbenchmark(grow(), preallocate(), vectorise())
print(b, unit = "eps")

# Key to this technique is building up a good R vocabulary

# If you don't know how long it will be, periodically 
# double the length


grow2 <- function() {
  set.seed(1000)
  
  output <- numeric()
  while(sample(1e5, 1) > 1) {
    output <- c(output, 1)
  }
  output
}
double <- function() {
  set.seed(1000)

  output <- rep(NA, 10)
  n <- 10
  i <- 0
  
  while(sample(1e5, 1) > 1) {
    i <- i + 1
    if (i > n) {
      output <- c(output, rep(NA, n))
      n <- 2 * n
    }
    output[i] <- 1
  }
  output[seq_len(i)]
}

system.time(g <- grow2())
system.time(d <- double())
all.equal(d, g)


# Always work with the simplest possible data structure. 
# Avoid modifying data.frames in a loop
df <- function() {
  for(i in nrow(mtcars)) {
    mtcars[i, "cyl"] <- mtcars[i, "cyl"] * 2
  }
  
  mtcars
}

vector <- function() {
  var <- mtcars$cyl
  
  for(i in nrow(mtcars)) {
    var[i] <- var[i] * 2
  }
  mtcars$cyl <- var
  mtcars
}

b <- microbenchmark(df(), vector())
print(b, unit = "eps")

# Byte code compiler -----------------------------------------------------

library(compiler)

grow_c <- cmpfun(grow)
preallocate_c <- cmpfun(preallocate)

b <- microbenchmark(grow(), grow_c(), preallocate(), preallocate_c())
print(b, unit = "eps")

grow2_c <- cmpfun(grow2)
double_c <- cmpfun(double)
system.time(g <- grow2_c())
system.time(d <- double_c())

